@@ -90,24 +90,20 @@ register_backup(struct pkgdb *db, int fd, const char *path)
 	return (retcode);
 }
 
-void
+static void
 backup_library(struct pkgdb *db, struct pkg *p, const char *path)
 {
 	const char *libname;
-	char buf[BUFSIZ];
-	char *outbuf;
 	int from, to, backupdir;
-	ssize_t nread, nwritten;
 
 	if ((libname = strrchr(path, '/')) == NULL)
 		return;
+	/* skip the initial / */
+	libname++;
 
 	pkg_open_root_fd(p);
 	to = -1;
 
-	/* skip the initial / */
-	libname++;
-
 	from = openat(p->rootfd, RELATIVE_PATH(path), O_RDONLY);
 	if (from == -1) {
 		pkg_debug(2, "unable to backup %s:%s", path, strerror(errno));
@@ -142,21 +138,7 @@ backup_library(struct pkgdb *db, struct pkg *p, const char *path)
 		goto out;
 	}
 
-	memset(buf, '\0', sizeof(buf));
-	while ((nread = read(from, buf, sizeof(buf))) > 0) {
-		outbuf = buf;
-		do {
-			nwritten = write(to, outbuf, nread);
-			if (nwritten >= 0) {
-				nread -= nwritten;
-				outbuf += nwritten;
-			} else if (errno != EINTR) {
-				goto out;
-			}
-		} while (nread > 0);
-	}
-
-	if (nread == 0) {
+	if (pkg_copy_file(from, to)) {
 		if (close(to) < 0) {
 			to = -1;
 			goto out;
@@ -167,7 +149,6 @@ backup_library(struct pkgdb *db, struct pkg *p, const char *path)
 		return;
 	}
 
-
 out:
 	pkg_emit_errno("Fail to backup the library", libname);
 	if (backupdir >= 0)
@@ -177,3 +158,22 @@ out:
 	if (to >= 0)
 		close(to);
 }
+
+/*
+ * We're about to remove an installed file as part of an upgrade.  See if it's a
+ * library and whether the user asked us to back up libraries, and if so, back
+ * it up.
+ */
+void
+pkg_maybe_backup_library(struct pkgdb *db, struct pkg *pkg, const char *path)
+{
+	const char *libname;
+
+	if (!ctx.backup_libraries)
+		return;
+
+	libname = strrchr(path, '/');
+	if (libname != NULL &&
+	    charv_search(&pkg->shlibs_provided, libname + 1) != NULL)
+		backup_library(db, pkg, path);
+}
