@@ -153,6 +153,37 @@ pkg_kv_free(struct pkg_kv *c)
 	free(c);
 }
 
+static int
+kv_cmp(const void *a, const void *b) {
+	struct pkg_kv *ka = *(struct pkg_kv **)a;
+	struct pkg_kv *kb = *(struct pkg_kv **)b;
+
+	return (strcmp(ka->key, kb->key));
+}
+
+struct pkg_kv *
+pkg_kv_search(kvlist_t *kv, char *el)
+{
+	struct pkg_kv target =  { .key = el, .value = NULL };
+	struct pkg_kv *tgt = &target;
+	if (kv->len == 0)
+		return (NULL);
+	struct pkg_kv **res = bsearch(&tgt, kv->d, kv->len, sizeof(kv->d[0]), kv_cmp);
+	if (res == NULL)
+		return (NULL);
+	return (*res);
+}
+
+DEFINE_VEC_INSERT_SORTED_FUNC(kvlist_t, pkg_kv, struct pkg_kv *, kv_cmp)
+
+void
+pkg_kv_sort(kvlist_t *kv)
+{
+	if (kv->len == 0)
+		return;
+	qsort(kv->d, kv->len, sizeof(kv->d[0]), kv_cmp);
+}
+
 struct pkg_kvlist_iterator *
 pkg_kvlist_iterator(struct pkg_kvlist *l)
 {
