@@ -1,29 +1,9 @@
 /*-
- * Copyright (c) 2011-2023 Baptiste Daroussin <bapt@FreeBSD.org>
+ * Copyright (c) 2011-2025 Baptiste Daroussin <bapt@FreeBSD.org>
  * Copyright (c) 2011-2012 Julien Laffaye <jlaffaye@FreeBSD.org>
  * Copyright (c) 2013-2014 Vsevolod Stakhov <vsevolod@FreeBSD.org>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer
- *    in this position and unchanged.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <sys/types.h>
@@ -240,44 +220,29 @@ urlencode(const char *src, xstring **dest)
 	return (EPKG_OK);
 }
 
-static int
-urldecode(const char *src, xstring **dest)
+static char*
+url_decode(const char* src, size_t len)
 {
-	size_t len;
-	size_t i;
-	char c;
-	char hex[] = {'\0', '\0', '\0'};
+	if (len == 0)
+		len = strlen(src);
 
-	xstring_renew(*dest);
-
-	len = strlen(src);
-	for (i = 0; i < len; i++) {
-		if (src[i] != '%') {
-			fputc(src[i], (*dest)->fp);
+	char* dest = xmalloc(len + 1);
+	char* p = dest;
+	for (size_t i = 0; i < len; i++) {
+		if (src[i] == '%') {
+			if (i + 2 < len) {
+				int value;
+				sscanf(src + i + 1, "%2x", &value);
+				*p++ = (char)value;
+				i += 2;
+			}
 		} else {
-			if (i + 2 > len) {
-				pkg_emit_error("unexpected end of string");
-				return (EPKG_FATAL);
-			}
-
-			hex[0] = src[++i];
-			hex[1] = src[++i];
-			errno = 0;
-			c = strtol(hex, NULL, 16);
-			if (errno != 0) {
-				/*
-				 * if it fails consider this is not a urlencoded
-				 * information
-				 */
-				fprintf((*dest)->fp, "%%%s", hex);
-			} else {
-				fputc(c,(*dest)->fp);
-			}
+			*p++ = src[i];
 		}
 	}
+	*p = '\0';
 
-	fflush((*dest)->fp);
-	return (EPKG_OK);
+	return (dest);
 }
 
 static int
@@ -316,8 +281,8 @@ static int
 pkg_string(struct pkg *pkg, const ucl_object_t *obj, uint32_t offset)
 {
 	const char *str;
+	char *tofree = NULL;
 	char **dest;
-	xstring *buf = NULL;
 
 	str = ucl_object_tostring_forced(obj);
 
@@ -338,8 +303,8 @@ pkg_string(struct pkg *pkg, const ucl_object_t *obj, uint32_t offset)
 	else {
 
 		if (offset & STRING_FLAG_URLDECODE) {
-			urldecode(str, &buf);
-			str = buf->buf;
+			tofree = url_decode(str, 0);
+			str = tofree;
 		}
 
 		/* Remove flags from the offset */
@@ -347,7 +312,7 @@ pkg_string(struct pkg *pkg, const ucl_object_t *obj, uint32_t offset)
 		dest = (char **) ((unsigned char *)pkg + offset);
 		*dest = xstrdup(str);
 
-		xstring_free(buf);
+		free(tofree);
 	}
 
 	return (EPKG_OK);
@@ -471,7 +436,6 @@ pkg_array(struct pkg *pkg, const ucl_object_t *obj, uint32_t attr)
 static int
 pkg_obj(struct pkg *pkg, const ucl_object_t *obj, uint32_t attr)
 {
-	xstring *tmp = NULL;
 	const ucl_object_t *cur;
 	ucl_object_iter_t it = NULL;
 	pkg_script script_type;
@@ -501,13 +465,15 @@ pkg_obj(struct pkg *pkg, const ucl_object_t *obj, uint32_t attr)
 			break;
 		case MANIFEST_DIRECTORIES:
 			if (cur->type == UCL_BOOLEAN) {
-				urldecode(key, &tmp);
-				pkg_adddir(pkg, tmp->buf, false);
+				char *t = url_decode(key, 0);
+				pkg_adddir(pkg, t, false);
+				free(t);
 			} else if (cur->type == UCL_OBJECT) {
 				pkg_set_dirs_from_object(pkg, cur);
 			} else if (cur->type == UCL_STRING) {
-				urldecode(key, &tmp);
-				pkg_adddir(pkg, tmp->buf, false);
+				char *t = url_decode(key, 0);
+				pkg_adddir(pkg, t, false);
+				free(t);
 			} else {
 				pkg_emit_error("Skipping malformed directories %s",
 				    key);
@@ -515,9 +481,10 @@ pkg_obj(struct pkg *pkg, const ucl_object_t *obj, uint32_t attr)
 			break;
 		case MANIFEST_FILES:
 			if (cur->type == UCL_STRING) {
+				char *t = url_decode(key, 0);
 				buf = ucl_object_tolstring(cur, &len);
-				urldecode(key, &tmp);
-				pkg_addfile(pkg, tmp->buf, len >= 2 ? buf : NULL, false);
+				pkg_addfile(pkg, t, len >= 2 ? buf : NULL, false);
+				free(t);
 			} else if (cur->type == UCL_OBJECT)
 				pkg_set_files_from_object(pkg, cur);
 			else
@@ -562,8 +529,9 @@ pkg_obj(struct pkg *pkg, const ucl_object_t *obj, uint32_t attr)
 					break;
 				}
 
-				urldecode(ucl_object_tostring(cur), &tmp);
-				pkg_addscript(pkg, tmp->buf, script_type);
+				char *t = url_decode(ucl_object_tostring(cur), 0);
+				pkg_addscript(pkg, t, script_type);
+				free(t);
 			}
 			break;
 		case MANIFEST_LUA_SCRIPTS:
@@ -590,8 +558,6 @@ pkg_obj(struct pkg *pkg, const ucl_object_t *obj, uint32_t attr)
 		}
 	}
 
-	xstring_free(tmp);
-
 	return (EPKG_OK);
 }
 
@@ -610,13 +576,13 @@ pkg_set_files_from_object(struct pkg *pkg, const ucl_object_t *obj)
 	const char *uname = NULL;
 	const char *gname = NULL;
 	mode_t perm = 0;
-	xstring *fname = NULL;
+	char *fname = NULL;
 	const char *key, *okey;
 
 	okey = ucl_object_key(obj);
 	if (okey == NULL)
 		return (EPKG_FATAL);
-	urldecode(okey, &fname);
+	fname = url_decode(okey, 0);
 	while ((cur = ucl_iterate_object(obj, &it, true))) {
 		key = ucl_object_key(cur);
 		if (key == NULL)
@@ -639,13 +605,13 @@ pkg_set_files_from_object(struct pkg *pkg, const ucl_object_t *obj)
 			free(set);
 		} else {
 			dbg(1, "Skipping unknown key for file(%s): %s",
-			    fname->buf, key);
+			    fname, key);
 		}
 	}
 
-	pkg_addfile_attr(pkg, fname->buf, sum, uname, gname, perm, 0,
+	pkg_addfile_attr(pkg, fname, sum, uname, gname, perm, 0,
 	    false);
-	xstring_free(fname);
+	free(fname);
 
 	return (EPKG_OK);
 }
@@ -658,13 +624,13 @@ pkg_set_dirs_from_object(struct pkg *pkg, const ucl_object_t *obj)
 	const char *uname = NULL;
 	const char *gname = NULL;
 	mode_t perm = 0;
-	xstring *dirname = NULL;
+	char *dirname = NULL;
 	const char *key, *okey;
 
 	okey = ucl_object_key(obj);
 	if (okey == NULL)
 		return (EPKG_FATAL);
-	urldecode(okey, &dirname);
+	dirname = url_decode(okey, 0);
 	while ((cur = ucl_iterate_object(obj, &it, true))) {
 		key = ucl_object_key(cur);
 		if (key == NULL)
@@ -686,12 +652,12 @@ pkg_set_dirs_from_object(struct pkg *pkg, const ucl_object_t *obj)
 			/* ignore on purpose : compatibility*/
 		} else {
 			dbg(1, "Skipping unknown key for dir(%s): %s",
-			    dirname->buf, key);
+			    dirname, key);
 		}
 	}
 
-	pkg_adddir_attr(pkg, dirname->buf, uname, gname, perm, 0, false);
-	xstring_free(dirname);
+	pkg_adddir_attr(pkg, dirname, uname, gname, perm, 0, false);
+	free(dirname);
 
 	return (EPKG_OK);
 }
@@ -704,9 +670,7 @@ pkg_set_deps_from_object(struct pkg *pkg, const ucl_object_t *obj)
 	const char *origin = NULL;
 	const char *version = NULL;
 	const char *key, *okey;
-	bool noversion = false;
 
-	noversion = (getenv("PKG_NO_VERSION_FOR_DEPS") != NULL);
 	okey = ucl_object_key(obj);
 	if (okey == NULL)
 		return (EPKG_FATAL);
@@ -720,7 +684,7 @@ pkg_set_deps_from_object(struct pkg *pkg, const ucl_object_t *obj)
 			if (cur->type != UCL_STRING) {
 				/* accept version to be an integer */
 				if (cur->type == UCL_INT && STRIEQ(key, "version")) {
-					if (!noversion)
+					if (!ctx.no_version_for_deps)
 						version = ucl_object_tostring_forced(cur);
 					continue;
 				}
@@ -731,7 +695,7 @@ pkg_set_deps_from_object(struct pkg *pkg, const ucl_object_t *obj)
 			}
 			if (STRIEQ(key, "origin"))
 				origin = ucl_object_tostring(cur);
-			if (STRIEQ(key, "version") && !noversion)
+			if (STRIEQ(key, "version") && !ctx.no_version_for_deps)
 				version = ucl_object_tostring(cur);
 		}
 		if (origin != NULL)
@@ -999,10 +963,10 @@ pkg_emit_object(struct pkg *pkg, short flags)
 
 	dbg(4, "Emitting licenses");
 	seq = NULL;
-	tll_foreach(pkg->licenses, l) {
+	vec_foreach(pkg->licenses, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(l->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->licenses.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "licenses", 8, false);
@@ -1034,50 +998,50 @@ pkg_emit_object(struct pkg *pkg, short flags)
 
 	dbg(4, "Emitting categories");
 	seq = NULL;
-	tll_foreach(pkg->categories, c) {
+	vec_foreach(pkg->categories, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(c->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->categories.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "categories", 10, false);
 
 	dbg(4, "Emitting users");
 	seq = NULL;
-	tll_foreach(pkg->users, u) {
+	vec_foreach(pkg->users, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(u->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->users.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "users", 5, false);
 
 	dbg(4, "Emitting groups");
 	seq = NULL;
-	tll_foreach(pkg->groups, g) {
+	vec_foreach(pkg->groups, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(g->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->groups.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "groups", 6, false);
 
 	dbg(4, "Emitting shibs_required");
 	seq = NULL;
-	tll_foreach(pkg->shlibs_required, s) {
+	vec_foreach(pkg->shlibs_required, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(s->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->shlibs_required.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "shlibs_required", 15, false);
 
 	dbg(4, "Emitting shlibs_provided");
 	seq = NULL;
-	tll_foreach(pkg->shlibs_provided, s) {
+	vec_foreach(pkg->shlibs_provided, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(s->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->shlibs_provided.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "shlibs_provided", 15, false);
@@ -1094,20 +1058,20 @@ pkg_emit_object(struct pkg *pkg, short flags)
 
 	dbg(4, "Emitting provides");
 	seq = NULL;
-	tll_foreach(pkg->provides, p) {
+	vec_foreach(pkg->provides, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(p->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->provides.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "provides", 8, false);
 
 	dbg(4, "Emitting requires");
 	seq = NULL;
-	tll_foreach(pkg->requires, r) {
+	vec_foreach(pkg->requires, i) {
 		if (seq == NULL)
 			seq = ucl_object_typed_new(UCL_ARRAY);
-		ucl_array_append(seq, ucl_object_fromstring(r->item));
+		ucl_array_append(seq, ucl_object_fromstring(pkg->requires.d[i]));
 	}
 	if (seq)
 		ucl_object_insert_key(top, seq, "requires", 8, false);
@@ -1126,8 +1090,8 @@ pkg_emit_object(struct pkg *pkg, short flags)
 		ucl_object_insert_key(top, map, "options", 7, false);
 
 	map = NULL;
-	tll_foreach(pkg->annotations, k) {
-		kv = k->item;
+	vec_foreach(pkg->annotations, i) {
+		kv = pkg->annotations.d[i];
 		if (map == NULL)
 			map = ucl_object_typed_new(UCL_OBJECT);
 		/* Add annotations except for internal ones. */
@@ -1236,7 +1200,7 @@ pkg_emit_object(struct pkg *pkg, short flags)
 		dbg(4, "Emitting lua scripts");
 		map = NULL;
 		for (i = 0; i < PKG_NUM_LUA_SCRIPTS; i++) {
-			if (tll_length(pkg->lua_scripts[i]) == 0)
+			if (vec_len(&pkg->lua_scripts[i]) == 0)
 				continue;
 			switch(i) {
 			case PKG_LUA_PRE_INSTALL:
